<?php
/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ldapmonitor 1.0.0
 * by Daif Alazmi (http://daif.net) - 2020-03-08
 * daif@daif.net
 * 
 * Link:
 *  https://daif.net/ldapmonitor/
 *
*/

define(VERSION, '1.0.0');
define(FOLDER, __DIR__);

error_reporting(E_ALL);
ini_set('display_errors', TRUE);
session_start();

$ldap_host = '10.10.10.10'; // e.g ldap://10.10.10.10
$ldap_conn = null;
$base_path = 'http://localhost/ldapmonitor/';

// check if ldap extension is loaded 
if(!extension_loaded('ldap'))
{
	print '<b>Error:</b> php7.2-ldap extension is not installed.';
	die;
}

function BaseURL($path='')
{
	global $base_path;
	return(rtrim($base_path,'/').'/'.ltrim($path, '/'));
}

function Config($config=[])
{
	if(!empty($config))
	{
		$config = json_encode($config, JSON_PRETTY_PRINT);
		return file_put_contents(FOLDER.'/.config.json', $config);
	}
	return json_decode(file_get_contents(FOLDER.'/.config.json'), true);
}

function UserLoginCheck()
{
	global $ldap_host, $ldap_conn, $ldap_bind;
	if(!empty($_POST['user']) && !empty($_POST['password']))
	{
		unset($_SESSION['user']);
		$ldap_conn = ldap_connect($ldap_host);
		ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, 3);
		ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0);
		$ldap_bind = ldap_bind($ldap_conn, $_POST['user'], $_POST['password']);
		if($ldap_bind)
		{
			$_SESSION['user'] = ['user'=>$_POST['user'],'password'=>$_POST['password']];
			header('location: '.BaseURL(''));
			exit;
		}
		else
		{
			$_SESSION['error'] = 'login failed';
		}
	}
	if(isset($_SESSION['user']['user']) && isset($_SESSION['user']['password']))
	{
		return TRUE;
	}
	return FALSE;
}

// search in LDAP
function LdapSearch($search, $conn=false)
{
	global $ldap_host, $ldap_conn, $ldap_bind;
	// cache 
	$search['update'] = 60*(int)$search['update'];
	if(($data = CacheFile($search['filter'], NULL, $search['update'])) != false)
	{
		return $data;
	}
	// data array
	$data = [];
	// connect for first 
	$ldap_conn = ldap_connect($ldap_host);
	ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, 3);
	ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0);
	$ldap_bind = ldap_bind($ldap_conn, $_SESSION['user']['user'], $_SESSION['user']['password']);
	if($ldap_bind)
	{
		$ldap_info = ldap_info();
		if(empty($search['attributes']))	$search['attributes'] 	= ['*'];
		if(empty($search['base_dn'])) 		$search['base_dn'] 		= $ldap_info['distinguishedname'];
		// replace vars, now | -5 day | +1 week | +1 week 2 days 4 hours 2 seconds , php.net/strtotime
		$search['filter'] = preg_replace_callback('/(whencreated|whenchanged)([=<>]+){date:(.+)}/Uis', function ($matches){
			return $matches[1].$matches[2].date('YmdHis', strtotime($matches[3])).'.0Z';
		}, $search['filter']);
		$search['filter'] = preg_replace_callback('/{date:(.+)}/Uis', function ($matches) {
			return UnixToWin(date('Y-m-d H:i:s', strtotime($matches[1])));
		}, $search['filter']);
		// search
		$result = @ldap_search($ldap_conn, $search['base_dn'], $search['filter'], $search['attributes']);
		if($result === FALSE)
		{
			return FALSE;
		}
		$result = ldap_get_entries($ldap_conn, $result);
		// re-arrange the array
		foreach ($result as $key => $row)
		{
			if(!is_array($row)) continue;
			foreach ($row as $attr => $val)
			{
				if(!is_array($val)) continue;
				$data[$key][$attr] = $val[0];
			}
		}
		// sort the array with sort column 
		if(isset($search['sort']) && $search['sort'] !== '')
		{
			usort($data, arraySortByKey($search['sort']));
		}
		$data[0]['updated_at'] = date('Y-m-d H:i:s');
		// save in cache
		CacheFile($search['filter'], $data);
	}
	return $data;
}

function ldap_info()
{
	global $ldap_host, $ldap_conn, $ldap_bind;
	static $ldap_info;
	if(empty($ldap_info))
	{
		$result = @ldap_read($ldap_conn, '', '(objectclass=*)', ['*']);
		$result = @ldap_get_entries($ldap_conn, $result);
		if($result === FALSE)
		{
			return FALSE;
		}
		// find default base_dn
		if(!empty($result['0']['rootdomainnamingcontext']['0'])) 	$base_dn = $result['0']['rootdomainnamingcontext']['0'];
		if(!empty($result['0']['namingcontexts']['0'])) 			$base_dn = $result['0']['namingcontexts']['0'];
		if(!empty($result['0']['defaultnamingcontext']['0'])) 		$base_dn = $result['0']['defaultnamingcontext']['0'];
		if(empty($base_dn))
		{
			return FALSE;
		}
		$result = @ldap_read($ldap_conn, $base_dn, '(objectclass=*)', ['*']);
		$result = @ldap_get_entries($ldap_conn, $result);
		if(!is_array($result[0]))
		{
			return FALSE;
		}
		// re-arrange the array
		foreach ($result[0] as $key => $val)
		{
			if(!is_array($val)) continue;
			$ldap_info[$key] = $val[0];
		}
	}
	return $ldap_info;
}


// return HTML table from array 
function RowsToTable($rows, $cols=[])
{
	$data  = '<table class="table table-bordered">';
	$data .= '<thead>';
	$data .= '<tr>';
	foreach ($cols as $key => $col)
	{
		$data .= '<th scope="col">'.$col.'</th>';
	}
	$data .= '</tr>';
	$data .= '</thead>';
	$data .= '<tbody>';
	foreach ($rows as $key => $row)
	{
		$data .= '<tr>';
		foreach ($cols as $key => $col)
		{
			$data .= '<td>'.printRow($row, $col).'</td>';
		}
		$data .= '</tr>';
	}
	$data .= '</tbody>';
	$data .= '</table>';
	return $data;
}

// format and print the data
function printRow($row, $key)
{
	$data = isset($row[strtolower($key)])?$row[strtolower($key)]:'';
	// ignore un expired date 
	if($data == '9223372036854775807') $data = 'Null';
	// windows timeStamp length is 18 integers, 131922053843891958
	if(preg_match('#\\d{18}#', $data))     $data = date('Y-m-d h:i:s', WinToUnix($data));
	if(preg_match('#\\d{14}.0Z#', $data))  $data = date('Y-m-d h:i:s', strtotime(str_replace('.0Z', '', $data)));
	return $data;
}

// sort array by key
function arraySortByKey($key, $order = 'DESC')
{
	$key = strtolower($key);
	return function ($a, $b) use ($key, $order)
	{
		if ($order == 'DESC')
		{
			list($a, $b) = [$b, $a];
		}
		return @strnatcmp($a[$key], $b[$key]);
	};
}

// convert windows timestamps to unix timestamps
function WinToUnix($TimeStamp)
{
	return ($TimeStamp/(10*1000*1000))-11644473600; 
}

// convert unix timestamps to windows timestamps
function UnixToWin($TimeStamp)
{
	return (strtotime($TimeStamp) * 10000000) + 116444736000000000;
}

// Tiny file based caching system 
function CacheFile($id, $data=NULL, $cache_expire=300) // 60*5
{
	$cache_path = FOLDER.'/.cache';
	$cache_file = $cache_path.'/.'.md5($id).'.cache';
	if (!file_exists($cache_path))
	{
		mkdir($cache_path, 0775, true);
	}
	// remove old files, age more than 60*60*24 
	$files = glob($cache_path.'/.*.cache');
	foreach ($files as $key => $file)
	{
		if(time() - filemtime($file) >= 60*60*24) unlink($file);
	}
	if($data)
	{
		return file_put_contents($cache_file, serialize($data));
	}
	elseif(file_exists($cache_file) && filemtime($cache_file) + $cache_expire > time())
	{
		return unserialize(file_get_contents($cache_file));
	}
	return NULL;
}