<?php
/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * ldapmonitor 1.0.0
 * by Daif Alazmi (http://daif.net) - 2020-03-08
 * daif@daif.net
 * 
 * Link:
 *  https://daif.net/ldapmonitor/
 *
*/

include(__DIR__.'/config.php');
$action = trim(substr($_SERVER['REQUEST_URI'], strlen(dirname($_SERVER['SCRIPT_NAME']))), '/');

//user logout
if($action == 'logout')
{
	unset($_SESSION['user']);
	header('location: '.BaseURL());
	exit;
}

// load reports
$reports = Config();

if(UserLoginCheck())
{
	// return count, badPasswordTimeToday
	if($action == 'api/count')
	{
		header('Content-Type: application/json');
		if(empty($_POST['id']) || !isset($reports[$_POST['id']]))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'404 Not Found']);
			exit;
		}
		$rows 	= LdapSearch($reports[$_POST['id']]);
		if(empty($rows))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'Report is not found.']);
			exit;
		}
		// print data 
		header('HTTP/1.1 200 OK');
		print json_encode(['status'=>'200', 'id'=>$_POST['id'], 'updated_at'=>$rows['0']['updated_at'], 'count'=>count($rows)]);
		exit;
	}
	// return list if data
	if($action == 'api/list')
	{
		header('Content-Type: application/json');
		if(empty($_POST['id']) || !isset($reports[$_POST['id']]))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'404 Not Found']);
			exit;
		}
		$rows	= LdapSearch($reports[$_POST['id']]);
		if(empty($rows))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'Report is not found.']);
			exit;
		}
		$data   = RowsToTable($rows, $reports[$_POST['id']]['columns']);

		// print data 
		header('HTTP/1.1 200 OK');
		print json_encode(['status'=>'200', 'id'=>$_POST['id'], 'updated_at'=>$rows['0']['updated_at'], 'data'=>$data]);
		exit;
	}
	// return list if data, badPasswordTimeToday/list
	if($action == 'api/edit')
	{
		header('Content-Type: application/json');
		if(empty($_POST['id']) || !isset($reports[$_POST['id']]))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'404 Not Found']);
			exit;
		}
		$data   = $reports[$_POST['id']];

		// print data 
		header('HTTP/1.1 200 OK');
		print json_encode(['status'=>'200', 'id'=>$_POST['id'], 'data'=>$data]);
		exit;
	}
	// test and save the report
	if($action == 'api/save')
	{
		header('Content-Type: application/json');

		if(!empty($_POST['id']) && !isset($reports[$_POST['id']]))
		{
			header('HTTP/1.1 404 Not Found');
			print json_encode(['status'=>'404', 'message'=>'404 Not Found']);
			exit;
		}
		if(empty($_POST['id'])) $_POST['id'] = time();

		$_POST['columns'] = array_map('trim', explode(',', $_POST['columns']));
		$reports[$_POST['id']] = $_POST;
		unset($reports[$_POST['id']]['id']);

		if(!LdapSearch($reports[$_POST['id']]))
		{
			header('HTTP/1.1 406 Not Acceptable');
			print json_encode(['status'=>'406', 'message'=>'LDAP query syntax is incorrect.']);
			exit;
		}

		$data = $reports[$_POST['id']];
		// save
		Config($reports);

		// print data 
		header('HTTP/1.1 200 OK');
		print json_encode(['status'=>'200', 'id'=>$_POST['id'], 'data'=>$data]);
		exit;
	}
	// delete the report
	if($action == 'api/delete')
	{
		header('Content-Type: application/json');
		// delete 
		if(isset($reports[$_POST['id']]))
		{
			unset($reports[$_POST['id']]);
			// save
			Config($reports);
		}
		// print data 
		header('HTTP/1.1 200 OK');
		print json_encode(['status'=>'200', 'message'=>'report deleted']);
		exit;
	}

	if($action == '')
	{
		// set default value
		foreach ($reports as $key => $report)
		{
			$reports[$key]['id'] 			= $key;
			$reports[$key]['count'] 		= '0';
			$reports[$key]['updated_at']	= '00:00:00';
		}
	}

}

include(__DIR__.'/layout.php');
?>